﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Menus;
using Infragistics.Windows;

namespace DataTreeDragDropFramework
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Fields

        private bool dropBefore = false;
        private bool dropAfter = false;
        private XamDataTreeNode parent = null;

        #endregion

        public MainWindow()
        {
            InitializeComponent();
        }

        #region Drag Drop Event Handlers

        private void DragSource_DragOver(object sender, Infragistics.DragDrop.DragDropMoveEventArgs e)
        {
            XamDataTreeNodeControl drop = Utilities.GetAncestorFromType(e.DropTarget, typeof(XamDataTreeNodeControl), false) as XamDataTreeNodeControl;
            XamDataTreeNodeControl drag = Utilities.GetAncestorFromType(e.DragSource, typeof(XamDataTreeNodeControl), false) as XamDataTreeNodeControl;

            if (drop != null && drop != null && drag.Node.Manager.ParentNode != null && drop.Node.Manager.ParentNode != null)
            {
                if (drag.Node.Data.GetType().Equals(drop.Node.Data.GetType()) && 
                    drag.Node.Manager.ParentNode.Data.Equals(drop.Node.Manager.ParentNode.Data))
                {
                    parent = drag.Node.Manager.ParentNode;

                    if (e.GetPosition(e.DropTarget).Y < drop.ActualHeight/2)
                    {
                        (Utilities.GetDescendantFromName(drop, "DropBeforeElem") as Grid).Visibility = System.Windows.Visibility.Visible;
                        (Utilities.GetDescendantFromName(drop, "DropAfterElem") as Grid).Visibility = System.Windows.Visibility.Collapsed;
                        dropBefore = true;
                        dropAfter = false;
                    }
                    else
                    {
                        (Utilities.GetDescendantFromName(drop, "DropAfterElem") as Grid).Visibility = System.Windows.Visibility.Visible;
                        (Utilities.GetDescendantFromName(drop, "DropBeforeElem") as Grid).Visibility = System.Windows.Visibility.Collapsed;
                        dropBefore = false;
                        dropAfter = true;
                    }  
                }
                
            }


        }

        private void DragSource_DragLeave(object sender, Infragistics.DragDrop.DragDropEventArgs e)
        {
            XamDataTreeNodeControl drop = Utilities.GetAncestorFromType(e.DropTarget, typeof(XamDataTreeNodeControl), false) as XamDataTreeNodeControl;
            Reset(drop);
        }

        private void DragSource_Drop(object sender, Infragistics.DragDrop.DropEventArgs e)
        {
            XamDataTreeNodeControl drop = Utilities.GetAncestorFromType(e.DropTarget, typeof(XamDataTreeNodeControl), false) as XamDataTreeNodeControl;
            XamDataTreeNodeControl drag = Utilities.GetAncestorFromType(e.DragSource, typeof(XamDataTreeNodeControl), false) as XamDataTreeNodeControl;

            if (drop != null && drop != null && drag.Node.Manager.ParentNode != null && drop.Node.Manager.ParentNode != null)
            {
                int dropIndex = drop.Node.Index;
                if (dropBefore)
                {
                    parent.Nodes.RemoveAt(drag.Node.Index);
                    if (dropIndex == 0)
                    {
                        parent.Nodes.Insert(0, drag.Node);
                    }
                    else
                    {
                        parent.Nodes.Insert(dropIndex, drag.Node);
                    }

                }

                if (dropAfter)
                {
                    parent.Nodes.RemoveAt(drag.Node.Index);
                    if (dropIndex == parent.Nodes.Count)
                    {
                        parent.Nodes.Add(drag.Node);
                    }
                    else
                    {
                        parent.Nodes.Insert(drop.Node.Index + 1, drag.Node);
                    }
                }

                Reset(drop);
            }
        }

        #endregion

        private void Reset(XamDataTreeNodeControl drop)
        {
            if (drop != null)
            {
                (Utilities.GetDescendantFromName(drop, "DropBeforeElem") as Grid).Visibility = System.Windows.Visibility.Collapsed;
                (Utilities.GetDescendantFromName(drop, "DropAfterElem") as Grid).Visibility = System.Windows.Visibility.Collapsed;
                dropBefore = false;
                dropAfter = false;
                parent = null;
            }
        }
    }

    #region Data
    public class ContinentCollection : ObservableCollection<Continent>
    {
        public ContinentCollection()
        {
            Add(
                new Continent
                {
                    Name = "Africa",
                    Countres = new ObservableCollection<Country>()
                    {
                        new Country()
                        {
                            Name = "Algeria",
                            Cities = new ObservableCollection<City>()
                            {
                                new City {Name = "Algiers"},
                                new City {Name = "Oran"},
                            }
                        },
                        new Country()
                        {
                            Name = "Angola",
                            Cities = new ObservableCollection<City>()
                            {
                                new City {Name = "Soyo"},
                                new City {Name = "Ndalatando"},
                                new City {Name = "Ambriz"},
                            }
                        },
                        new Country()
                        {
                            Name = "Cape Verde",
                            Cities = new ObservableCollection<City>()
                            {
                                new City {Name = "Mindelo"},
                                new City {Name = "Assomada"},
                            }
                        }
                    }
                });

            Add(
                new Continent
                {
                    Name = "Asia",
                    Countres = new ObservableCollection<Country>()
                                    {
                                        new Country()
                                        {
                                            Name = "Cambodia",
                                            Cities = new ObservableCollection<City>()
                                            {
                                                new City {Name = "Phnom Penh"},
                                                new City {Name = "Koh Kong"},
                                            }
                                        },
                                        new Country()
                                        {
                                            Name = "China",
                                            Cities = new ObservableCollection<City>()
                                            {
                                                new City {Name = "Shanghai"},
                                                new City {Name = "Beijing"},
                                                new City {Name = "Tianjin"},
                                            }
                                        },
                                        new Country()
                                        {
                                            Name = "Japan",
                                            Cities = new ObservableCollection<City>()
                                            {
                                                new City {Name = "Tokyo"},
                                                new City {Name = "Yokohama"},
                                                new City {Name = "Sapporo"},
                                            }
                                        }
                                    }
                });

            Add(new Continent
            {
                Name = "Australia",
                Countres = new ObservableCollection<Country>()
                                    {
                                        new Country()
                                        {
                                            Name = "Australia",
                                            Cities = new ObservableCollection<City>()
                                            {
                                                new City {Name = "Sydney"},
                                                new City {Name = "Melbourne"},
                                                new City {Name = "Perth"},
                                            }
                                        }
                                    }
            });

            Add(new Continent
            {
                Name = "Europe",
                Countres = new ObservableCollection<Country>()
                                    {
                                        new Country()
                                        {
                                            Name = "Germany",
                                            Cities = new ObservableCollection<City>()
                                            {
                                                new City {Name = "Berlin"},
                                                new City {Name = "Hamburg"}
                                            }
                                        },
                                        new Country()
                                        {
                                            Name = "Italy",
                                            Cities = new ObservableCollection<City>()
                                            {
                                                new City {Name = "Milan"},
                                                new City {Name = "Naples"},
                                                new City {Name = "Rome"}
                                            }
                                        }
                                    }
            });
        }
    }

    public class Continent
    {
        public string Name { get; set; }
        public ObservableCollection<Country> Countres { get; set; }
    }

    public class Country
    {
        public string Name { get; set; }
        public ObservableCollection<City> Cities { get; set; }
    }

    public class City
    {
        public string Name { get; set; }
    }

    public class Operator
    {
        public int ID { get; set; }
        public string Name { get; set; }
    }
    #endregion  

}
